//define PWM pins for each color
#define RED           3
#define GREEN         5
#define BLUE          6

//colors value
int r, g, b;

//Possible operating states of the LED.
enum LedState {automatic, on, off};
enum ColorMode {set, process};

//and default value
LedState ledState = automatic;
ColorMode colorMode = set;

void setup() {
    Serial.begin(9600); //for HC-06 could be 38400
    pinMode(RED, OUTPUT);
    pinMode(GREEN, OUTPUT);
    pinMode(BLUE, OUTPUT);

    digitalWrite(RED, LOW);
    digitalWrite(GREEN, LOW);
    digitalWrite(BLUE, LOW);
}

void loop() {
  if (Serial.available() > 0) {
    //all serial data is treated as color data until the colorMode is set
    if (colorMode == process) {
      Serial.println("-----------------------------");
      Serial.println("Waiting color data");
      setRGBcolor(&r, &g, &b, &colorMode);
    }
    //receive other BT commands
    else {
      switchCaseBTcommand();
    }
  }
  //block of code controlling the LED
  switch (ledState) {
    case automatic:
      ledControl(RED, GREEN, BLUE, outDarkMotion);
      break;

    case on:
      ledControl(RED, GREEN, BLUE, true);
      break;

    case off:
      ledControl(RED, GREEN, BLUE, false);
      break;
  }
}

void setRGBcolor(int *r, int *g, int *b, ColorMode *mode) {
  const byte numChars = 32;
  char receivedChars[numChars];
  char tempChars[numChars];        // temporary array for use when parsing
  static byte ndx = 0;
  char startMarker = '<';
  char endMarker = '>';
  char delimiter = ':';
  char rc;
  static bool recvInProgress = false;
  bool newData = false;

  //get newData
  while (Serial.available() > 0 && newData == false) {
    rc = Serial.read();
    if (recvInProgress == true) {
      if (rc != endMarker) {
        receivedChars[ndx] = rc;
        ndx++;
        if (ndx >= numChars) {
          ndx = numChars - 1;
        }
      } else {
        receivedChars[ndx] = '\0'; // terminate the string
        recvInProgress = false;
        ndx = 0;
        newData = true;
      }
    }
    else if (rc == startMarker) {
      recvInProgress = true;
    }
  }

  //parsing newData
  if (newData == true) {

    strcpy(tempChars, receivedChars);
    char *strtokIndx;

    strtokIndx = strtok(tempChars, delimiter);     // get the first part - the string
    *r = atoi(strtokIndx); // copy it to messageFromPC
    strtokIndx = strtok(NULL, delimiter); // this continues where the previous call left off
    *g = atoi(strtokIndx);     // convert this part to an integer
    strtokIndx = strtok(NULL, delimiter);
    *b = atoi(strtokIndx);     // convert this part to an integer

    newData = false; //new data was recived, reset flag
    *mode = set; //color was set, change color mode to set
  }
}

void switchCaseBTcommand() {
  int val = Serial.parseInt();
  //switch commands received through BT
  switch (val) {
    /* can be any commands defined as an integer,
     * here just example.
     * const int AUTO = 0; const int ON = 1; and so on
    */
    case AUTO:
      ledState = automatic;
      break;

    case ON:
      ledState = on;
      break;

    case OFF:
      ledState = off;
      break;

    case SET_COLOR:
      colorMode = process;
      break;
  }
}

void ledControl(int red_pin, int green_pin, int blue_pin, bool ledPowerOn) {
  if (ledPowerOn == true) {
    analogWrite(red_pin, r);
    analogWrite(green_pin, g);
    analogWrite(blue_pin, b);
  } else {
    digitalWrite(red_pin, LOW);
    digitalWrite(green_pin, LOW);
    digitalWrite(blue_pin, LOW);
  }
}